"""Separate billing info from subscription table

Revision ID: 31db7612ce94
Revises: 6f2715afe6c9
Create Date: 2025-03-11 08:41:11.268593

"""

# revision identifiers, used by Alembic.
revision = '31db7612ce94'
down_revision = '6f2715afe6c9'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # The billing address was added earlier for future use and at the time of
    # this migration there is no way of actually storing the user's billing
    # address in these columns -- so they can be uncerimoniously dropped.
    op.execute("""
    ALTER TABLE subscription
        DROP COLUMN full_name,
        DROP COLUMN business_name,
        DROP COLUMN address_1,
        DROP COLUMN address_2,
        DROP COLUMN city,
        DROP COLUMN region,
        DROP COLUMN postcode,
        DROP COLUMN country,
        DROP COLUMN vat;

    CREATE TABLE billing_address (
        id serial PRIMARY KEY,
        user_id integer NOT NULL UNIQUE REFERENCES "user"(id) ON DELETE CASCADE,
        full_name text,
        business_name text,
        address_1 text,
        address_2 text,
        city text,
        region text,
        postcode text,
        country text,
        vat text
    );
    """)


def downgrade():
    op.execute("""
    ALTER TABLE subscription
        ADD COLUMN full_name text,
        ADD COLUMN business_name text,
        ADD COLUMN address_1 text,
        ADD COLUMN address_2 text,
        ADD COLUMN city text,
        ADD COLUMN region text,
        ADD COLUMN postcode text,
        ADD COLUMN country text,
        ADD COLUMN vat text;

    DROP TABLE billing_address;
    """)
