"""Expand invoice table

Revision ID: 351c45247a68
Revises: 28201884ffa5
Create Date: 2025-03-13 13:56:54.551394

"""

# revision identifiers, used by Alembic.
revision = '351c45247a68'
down_revision = '28201884ffa5'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # Products for reference:
	# (1, 'Amateur Hackers'),
	# (2, 'Typical Hackers'),
	# (3, 'Professional Hackers');

    op.execute("""
    ALTER TABLE invoice RENAME created TO issued;
    ALTER TABLE invoice RENAME cents TO subtotal;
    ALTER TABLE invoice RENAME valid_thru TO service_end;
    
    ALTER TABLE invoice
    ADD COLUMN product_id INTEGER REFERENCES "product"(id),
    ADD COLUMN interval payment_interval,
    ADD COLUMN service_start timestamp without time zone,
    ADD COLUMN currency payment_currency,
    ADD COLUMN tax_rate integer,
    ADD COLUMN tax_charged integer,
    ADD COLUMN total integer,
    ADD COLUMN payment_id text;

    -- Uncerimoniously drop nonsense invoices, e.g. from when subsidized users
    -- got a fake invoice
    DELETE FROM invoice
    WHERE subtotal NOT IN (200, 500, 1000, 2000, 5000, 10000);

    -- Populate new columns
    UPDATE invoice
    SET product_id = CASE
        WHEN subtotal IN (200, 2000) THEN 1
        WHEN subtotal IN (500, 5000) THEN 2
        WHEN subtotal IN (1000, 10000) THEN 3
    END,
    interval = CASE
        WHEN subtotal IN (200, 500, 1000) THEN 'MONTHLY'::payment_interval
        WHEN subtotal IN (2000, 5000, 10000) THEN 'ANNUALLY'::payment_interval
    END,
    service_start = issued,
    currency = 'USD',
    tax_charged = 0,
    total = subtotal;

    ALTER TABLE invoice
    ALTER COLUMN product_id SET NOT NULL,
    ALTER COLUMN interval SET NOT NULL,
    ALTER COLUMN service_start SET NOT NULL,
    ALTER COLUMN currency SET NOT NULL,
    ALTER COLUMN tax_charged SET NOT NULL,
    ALTER COLUMN total SET NOT NULL;

    ALTER TABLE invoice
    DROP COLUMN updated,
    DROP COLUMN source;
    """)


def downgrade():
    op.execute("""
    ALTER TABLE invoice RENAME issued TO created;
    ALTER TABLE invoice RENAME service_end TO valid_thru;
    ALTER TABLE invoice RENAME total TO cents;

    ALTER TABLE invoice
    ADD COLUMN updated timestamp without time zone,
    ADD COLUMN source character varying(256);

    UPDATE invoice SET
        updated = created,
        source = 'Unknown';

    ALTER TABLE invoice
    DROP COLUMN product_id,
    DROP COLUMN service_start,
    DROP COLUMN interval,
    DROP COLUMN currency,
    DROP COLUMN subtotal,
    DROP COLUMN tax_rate,
    DROP COLUMN tax_charged,
    DROP COLUMN payment_id;
    """)
