import sqlalchemy as sa
import sqlalchemy_utils as sau
from srht.database import Base
from enum import Enum

class PaymentInterval(Enum):
    monthly = "MONTHLY"
    annually = "ANNUALLY"

class PaymentCurrency(Enum):
    USD = 'USD'
    EUR = 'EUR'

class SubscriptionStatus(Enum):
    PENDING = 'PENDING'
    SETTLEMENT = 'SETTLEMENT'
    ACTIVE = 'ACTIVE'
    INACTIVE = 'INACTIVE'

class Subscription(Base):
    __tablename__ = "subscription"

    id = sa.Column(sa.Integer, primary_key=True)

    user_id = sa.Column(sa.Integer, sa.ForeignKey('user.id'))
    user = sa.orm.relationship('User',
            backref=sa.orm.backref('subscriptions'),
            foreign_keys=[user_id])

    created = sa.Column(sa.DateTime, nullable=False)
    updated = sa.Column(sa.DateTime, nullable=False)
    status = sa.Column(
            sau.ChoiceType(SubscriptionStatus, impl=sa.String()),
            nullable=False,
            server_default='PENDING')

    product_id = sa.Column(sa.Integer, sa.ForeignKey('product.id'))
    currency = sa.Column(
            sau.ChoiceType(PaymentCurrency, impl=sa.String()),
            nullable=False)
    interval = sa.Column(
            sau.ChoiceType(PaymentInterval, impl=sa.String()),
            nullable=False)
    autorenew = sa.Column(sa.Boolean, nullable=False)

class Product(Base):
    __tablename__ = "product"
    id = sa.Column(sa.Integer, primary_key=True)

class Invoice(Base):
    __tablename__ = 'invoice'
    id = sa.Column(sa.Integer, primary_key=True)
    invoice_no = sa.Column(sa.Integer, nullable=False)
    issued = sa.Column(sa.DateTime, nullable=False)
    user_id = sa.Column(sa.Integer, sa.ForeignKey('user.id'), nullable=False)
    user = sa.orm.relationship("User", backref="invoices")

    product_id = sa.Column(sa.Integer, sa.ForeignKey('product.id'))
    interval = sa.Column(
            sau.ChoiceType(PaymentInterval, impl=sa.String()),
            nullable=False)
    service_start = sa.Column(sa.DateTime, nullable=False)
    service_end = sa.Column(sa.DateTime, nullable=False)

    currency = sa.Column(
            sau.ChoiceType(PaymentCurrency, impl=sa.String()),
            nullable=False)
    subtotal = sa.Column(sa.Integer, nullable=False)
    tax_rate = sa.Column(sa.Integer)
    tax_charged = sa.Column(sa.Integer, nullable=False)
    total = sa.Column(sa.Integer, nullable=False)

    payment_id = sa.Column(sa.Text)

    def __repr__(self):
        return '<Invoice {} uid:{} ${:.2f}>'.format(
                self.id, self.user_id, self.total / 100)
